function demo_cmaes(varargin)
%                     data_file_ind, max_it, silent, k_folds, svm_kernel,...
%                     use_tsne, C, gamma, mirna_alpha, t, svm_metric, classifier,...
%                     nworkers, repr_type, repr_d, manifold_name)
%
% NOTE See 
%     aux/empty_params.m
% for a list of acceptable parameters.

rng('shuffle'); % XXX XXX XXX
p = parse_params(varargin{:});
p

[data_set, data_labels, data_fname] = fetch_data(p.data_file_ind);

if p.use_tsne
    ordering = strcat(data_fname, '.tsne');
    load(ordering, '-mat'); % Load into Y variable.
    [~,feat_order] = sort(Y);
    data_set = data_set(:, feat_order);
end

if strcmp(p.repr_name, 'manifold') || strcmp(p.repr_name, 'manifold2')
    % Load manifold.
    manifold_path = strcat('data/mirna2/full.data.', p.manifold_name);
    fprintf(1, 'Using manifold repr at %s\n', manifold_path);
    load(manifold_path, '-mat');
    repr = ReprManifold_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifoldperm')
    fprintf(1, 'Using randmanifoldperm repr\n');
    [Y, ~, ~] = fetch_embedded_data(p.data_file_ind, p.manifold_d, p.cached_manifold, false);
    if ~strcmp(p.manifold_name, 'tsne')
        error('Undefined for %s\n', p.manifold_name);
    end
    repr = ReprRandManifoldPerm_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'manifold2')
    % Load manifold.
    manifold_path = strcat('data/mirna2/full.data.', p.manifold_name);
    fprintf(1, 'Using manifold repr at %s\n', manifold_path);
    load(manifold_path, '-mat');
    repr = ReprManifold2_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifold2')
    fprintf(1, 'Using rand manifold repr\n');
    [X, ~] = fetch_data(p.data_file_ind);
    if strcmp(p.manifold_name, 'tsne')
        Y = tsne(X', [], p.manifold_d, 50, 30.0);
    elseif strcmp(p.manifold_name, 'sne')
        Y = sne(X', p.manifold_d, 30.0);
    else
        error('Unknown manifold_name %s\n', p.manifold_name);
    end
    repr = ReprRandManifold2_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifold3')
    fprintf(1, 'Using rand manifold3 repr\n');
    [X, ~] = fetch_data(p.data_file_ind);
    if strcmp(p.manifold_name, 'tsne')
        Y = tsne(X', [], p.manifold_d, 50, 30.0);
    elseif strcmp(p.manifold_name, 'sne')
        Y = sne(X', p.manifold_d, 30.0);
    else
        error('Unknown manifold_name %s\n', p.manifold_name);
    end
    repr = ReprRandManifold3_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifold')
    fprintf(1, 'Using randmanifold%d repr multipoint\n', p.manifold_d);
    [Y, ~, ~] = fetch_embedded_data(p.data_file_ind, p.manifold_d, p.cached_manifold, false);
    if ~strcmp(p.manifold_name, 'tsne')
        error('Undefined for %s\n', p.manifold_name);
    end
    repr = ReprRandManifold_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifold_weights')
    fprintf(1, 'Using randmanifold%d+weights repr\n', p.manifold_d);
    [Y, ~, ~] = fetch_embedded_data(p.data_file_ind, p.manifold_d, p.cached_manifold, false);
    % [Y, cm] = csne_wrapper(X', 1, true);
    if ~strcmp(p.manifold_name, 'tsne')
        error('Undefined for %s\n', p.manifold_name);
    end
    repr = ReprRandManifoldWeights_init(p.repr_d, Y, p.manifold_name);
elseif strcmp(p.repr_name, 'randmanifoldmp')
    fprintf(1, 'Using randmanifold%d repr multipoint\n', p.manifold_d);
    [Y, ~, cm] = fetch_embedded_data(p.data_file_ind, p.manifold_d, p.cached_manifold, true);
    % [Y, cm] = csne_wrapper(X', 1, true);
    if ~strcmp(p.manifold_name, 'tsne')
        error('Multipoint embeddings undefined for %s\n', p.manifold_name);
    end
    repr = ReprRandManifoldMP_init(p.repr_d, Y, p.manifold_name, cm);
elseif strcmp(p.repr_name, 'randmanifoldmp_weights')
    fprintf(1, 'Using randmanifold%d+weights repr multipoint\n', p.manifold_d);
    [Y, ~, cm] = fetch_embedded_data(p.data_file_ind, p.manifold_d, p.cached_manifold, true);
    % [Y, cm] = csne_wrapper(X', 1, true);
    if ~strcmp(p.manifold_name, 'tsne')
        error('Multipoint embeddings undefined for %s\n', p.manifold_name);
    end
    repr = ReprRandManifoldMPWeights_init(p.repr_d, Y, p.manifold_name, cm);
elseif strcmp(p.repr_name, 'rand_sne_perm')
    fprintf(1, 'Using rand sne + perm repr\n');
    [X, ~] = fetch_data(p.data_file_ind);
    if strcmp(p.manifold_name, 'tsne')
        Y = tsne(X', [], 1, 50, 30.0);
    elseif strcmp(p.manifold_name, 'sne')
        Y = sne(X', 1, 30.0);
    else
        error('Unknown manifold_name %s\n', p.manifold_name);
    end
    [~,feat_order] = sort(Y);
    data_set = data_set(:, feat_order);
    [~, D] = size(data_set);
    repr = ReprRandAndPerm_init(D, p.repr_d);
elseif strcmp(p.repr_name, 'randfixed')
    fprintf(1, 'Using rand fixed repr\n');
    [X, ~] = fetch_data(p.data_file_ind);
    if strcmp(p.manifold_name, 'tsne')
        Y = tsne(X', [], 1, 50, 30.0);
    elseif strcmp(p.manifold_name, 'sne')
        Y = sne(X', 1, 30.0);
    else
        error('Unknown manifold_name %s\n', p.manifold_name);
    end
    [~,feat_order] = sort(Y);
    data_set = data_set(:, feat_order);
    [~, D] = size(data_set);
    repr = ReprFixed_init(D);
else
    fprintf(1, 'Using fixed repr\n');
    [~, D] = size(data_set);
    % repr = ReprOhOne_init(D);
    % repr = ReprFixlen_init(D);
    repr = ReprFixed_init(D);
end

global GLOB_repr;
GLOB_repr = repr.name;

%------Run the demo.------------------------------------------------------
try
    if p.silent
        evalc('main_loop(repr, p.data_file_ind, data_set, data_labels, p)');
    else
        main_loop(repr, p.data_file_ind, data_set, data_labels, p);
    end
catch ME
   print_msg('Caught an error; saving workspace.');
   % save('cmaes_crash.mat');
   rethrow(ME);
end

end
