
function [fit acc] = fitness_k_fold(repr,X,data_set,data_labels,param)
% Returns fitness (which is mostly accuracy-based) and raw accuracy.

% X_data,X_labels,Z_data,Z_labels)

[ninstances dim] = size(data_set);
[N nfeats] = size(X);

if nfeats == 1
    fprintf(1, 'Warning: fitness_k_fold got a column vector instead of a row one. Transposing...\n');
    X = X';
    [N nfeats] = size(X);
end

use_svm_k_fold = (param.svm_kernel < 10);

if ~use_svm_k_fold

    fit = zeros(N, param.k);
    acc = zeros(N, param.k);
    
    % Prepare a vector denoting to which part of the data a single instance belongs,
    % i.e. [1 1 2 3 1 4 5 2], with k parts in mind.
    set_size = ninstances / param.k;
    indicator = [];
    for i = 1:set_size+1
        indicator = [indicator 1:param.k];
    end
    % indicator = repmat(1:k, 1, 1:set_size+1);
    indicator = indicator(1:ninstances);
    indicator = indicator(randperm(length(indicator)));
    
    for fold = 1:param.k
    
        train_data   = data_set(indicator ~= fold, :);
        train_labels = data_labels(indicator ~= fold);
        test_data    = data_set(indicator == fold, :);
        test_labels  = data_labels(indicator == fold);

        for i = 1:N 
            % Calculation of objective function for agent number 'i'
            indices = repr.feature_list(repr, X(i,:));
            [fit(i, fold) acc(i, fold)] = inner_loop(indices, repr, train_data, train_labels, test_data, test_labels, param);
        end
    
    end
    
    fit = mean(fit, 2);
    acc = mean(acc, 2);

else

    fit = zeros(1, N);
    acc = zeros(1, N);

    if param.nworkers > 1
        if matlabpool('size') == 0 % checking to see if my pool is already open
            fprintf(1, 'param.nworkers %d\n', param.nworkers);
            matlabpool('open', num2str(param.nworkers));
        end
        parfor i = 1:N 
            indices = repr.feature_list(repr, X(i,:));
            [fit(i) acc(i)] = inner_loop(indices, repr, data_set, data_labels, [], [], param);
        end
    else
        for i = 1:N 
            indices = repr.feature_list(repr, X(i,:));
            [fit(i) acc(i)] = inner_loop(indices, repr, data_set, data_labels, [], [], param);
        end
    end
end
[~, ind] = max(fit);
% fprintf(1, '%s best metrix score: %f\t(fit %f)\n', param.svm_metric, acc(ind), fit(ind));

end
