function [X, labels, cm] = fetch_embedded_data(data_file_ind, dims, cached, use_multipoint)
% Compute 1-D and 2-D t-SNE and multipoint and cache if necessary
%
% Args:
% -----
%    data_file_ind : int, data index (see fetch_data)
%    d : int, dimensionality (1-D or 2-D)
%    cached : bool, fetch cached embedding (debugging purposes)
%    use_multipoint : bool, use MP t-SNE or not

[X, labels, fpath] = fetch_data(data_file_ind);

if use_multipoint
    method_name = 'mptsne';
else
    method_name = 'tsne';
end

tsne_fpath = sprintf('%s/%s.%dd.%s', pwd, fpath, dims, method_name);

fprintf(1, '%s\n', tsne_fpath);

cm = [];
if cached
    if ~(exist(tsne_fpath, 'file') == 2)
        fprintf(1, 'Does not exist\n');
        [Y, cm] = csne_wrapper(X', dims, use_multipoint);
        save(tsne_fpath, 'Y', 'cm');
    end
    fprintf(1, 'Loading the embedding.\n');
    data_fname = tsne_fpath;
    s = load(data_fname, '-mat');
    X = s.Y'; % NOTE Transposition
    if use_multipoint
        cm = s.cm;
    end
else 
    fprintf(1, 'Computing the embedding.\n');
    [Y, cm] = csne_wrapper(X', dims, use_multipoint);
    X = Y';
end

[~,ndims] = size(X);
if ndims > 5
    X = X';
end

end
