function save_log(repr, param, history, optimizer_params, iter)

data_file_ind = param.data_file_ind;
optimizer_ind = param.optimizer_ind;
base_accuracy = param.base_accuracy;
max_it        = param.max_it;
svm_kernel    = param.svm_kernel;
k_folds       = param.k_folds;
nlast         = param.save_logs_niter;
c             = param.c;
gamma         = param.gamma;
svm_metric    = param.svm_metric;
mirna_alpha   = param.mirna_alpha;
classifier    = param.classifier;
repr_name     = repr.name;

write_headers  = (exist('iter', 'var') && (iter == nlast));

%--------Create log directory
if ~exist(param.logdir, 'dir')
    mkdir(param.logdir);
end

%--------Output partial logs every niter iterations.
if (mod(iter, nlast) ~= 0)
    return;
end

filename_prefix = strcat(param.logdir, '/',...
                         'data=', num2str(data_file_ind),...
                         '_optimizer=', num2str(optimizer_ind),...
                         '_cls=', classifier,...
                         '_maxit=', num2str(max_it),...
                         '_svmkernel=', num2str(svm_kernel),...
                         '_kfolds=', num2str(k_folds),...
                         '_C=', num2str(c),...
                         '_gamma=', num2str(gamma),...
                         '_svmmetric=', svm_metric,...
                         '_alpha=', num2str(mirna_alpha),...
                         '_repr=', repr_name);

[~, hostname] = system('hostname');
stats_filename = strcat(filename_prefix,...
                        '_', strrep(strrep(param.start_time, ' ', '_'), ':', '-'),...
                        '_', num2str(param.randi));

mean_acc = history.mean_acc;
best_acc = history.best_acc;
mean_fnum = history.mean_fnum;
best_fnum = history.best_fnum;
mean_threshold = history.mean_threshold;
best_indiv = history.best_indiv;

fid = fopen(strcat(stats_filename, '.csv'), 'a');

if write_headers

    % Write params
    fprintf(fid, '# log v1.0 (19.10.2016)\n');
    fprintf(fid, '# Main params\n');
    fprintf(fid, 'data_file_ind %d\toptimizer_ind %d\t classifier %s \tmax_it %d\t svm_kernel %d\t k_folds %d\n',...
            data_file_ind, optimizer_ind, classifier, max_it, svm_kernel, k_folds);

    fprintf(fid, evalc(['disp(param)']));
    fprintf(fid, '# Optimizer-specific params\n');
    fprintf(fid, '%s\n', struct2str(optimizer_params));

    fprintf(fid, '# Base accuracy (without reduction)\n%.5f\n', base_accuracy);

    fprintf(fid, '# Iteration 0 denotes mean values from all iterations\n');
    fprintf(fid, 'iter\tbest_raw\tmean_acc\tbest_acc\tmean_fnum\tbest_fnum\tmean_threshold\tbest_threshold\tbest_indiv_len\n');

end

    % Start with average and best values.
    % Find the best of all.
    bbest = best_acc*1000000 + (1./(best_fnum + 1));
    [~,i] = max(bbest);

if nlast == -1
    start_ = 1;
    end_ = max_it;
    end_ = min(end_, length(mean_acc));
    end_ = min(end_, length(best_acc));
    end_ = min(end_, length(mean_fnum));
    end_ = min(end_, length(best_fnum));
    end_ = min(end_, length(mean_threshold));
    [tmp_, ~] = size(best_indiv);
    end_ = min(end_, tmp_);
    % end_ = min(end_, length(best_indiv));
else
    assert(max_it >= nlast);
    start_ = iter-nlast+1;
    end_   = iter;
end

try
    for i = start_:end_
        best_raw = fitness2score(best_acc(i), best_fnum(i), param);
        fprintf(fid, '%d\t%.5f\t%.5f\t%.5f\t%d\t%.5f\t%.5f\t',...
                i, best_raw, mean_acc(i), best_acc(i), mean_fnum(i), best_fnum(i),...
                mean_threshold(i), best_indiv(i, end));
    
        % genes = (best_indiv(i, 1:end-1) > best_indiv(i,end));
        % genes = find(genes == 1);
        % fprintf(fid, '%d\t%s\n', i, mat2str(genes));
        fprintf(fid, '%d\t%s\n', i, mat2str(repr.gene_list(repr, best_indiv(i, :))));
    end
catch ME
end

inds = find(best_acc == max(best_acc));
mfnum = min(best_fnum(inds));
inds2 = find(best_fnum == mfnum);
inds = intersect(inds, inds2);
i = inds(end);

% B for best.
%%% fprintf(fid, 'B\t%.5f\t%.5f\t%.5f\t%d\t%.5f\t%.5f\t',...
%%%             mean_acc(i), best_acc(i), mean_fnum(i), best_fnum(i),...
%%%             mean_threshold(i), best_indiv(i, end));
%%% fprintf(fid, '%d\t%s\n', i, mat2str(repr.gene_list(repr, best_indiv(i, :))));

if iter == max_it
    % Save the best one as .mat
    mat_filename = strcat(stats_filename, '.mat');
    genes = repr.gene_list(repr, best_indiv(i,:));
    save(mat_filename, 'genes');

    % A for avg.
    fprintf(fid, 'A\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\t%.5f\n',...
            mean(mean_acc), mean(best_acc), mean(mean_fnum), mean(best_fnum),...
            mean(best_indiv(:, end)), mean(mean_threshold), mean(best_indiv(:, end)));

    % Write population data
    fprintf(fid, '# Raw best individual\n%s\n', mat2str(best_indiv(end, :)));

end
     
fclose(fid);

end % save_log

function score = fitness2score(fitness, nfeatures, param)
score = fitness + (max(0, nfeatures - param.t) * param.mirna_alpha);
end
