function [X_norm] = normalize_data(X, nbins)
% If nbins is given, the data will be fitted into [-0.5,nbins - 0.5] and divided into nbins, starting from 0.
% Otherwise it will return data of mean 0 varaince 1.

if nargin < 2
    % Just do mean 0 var 1 column-wise and return.
    X_norm = bsxfun(@minus, X', mean(X)')';
    X_norm = bsxfun(@rdivide, X_norm', std(X_norm)')';
    return % X_norm

    % Same as above, but matrix-wise.
    % X_norm = X - mean(X(:));
    % X_norm = X_norm / std(X_norm(:));
    % return % X_norm
end

% Fit to [-0.5, nbins-0.5] and then round.
X_norm = bsxfun(@minus, X', min(X,[],1)')';
X_norm = bsxfun(@rdivide, X_norm', max(X_norm,[],1)')';
X_norm = X_norm * (1.0 * nbins) - 0.5;
X_norm = round(X_norm);

% Make sure that after rounding all is fine.
X_norm = max(X_norm, 0);
X_norm = min(X_norm, nbins-1);

return % X_norm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Old code -- from before Dec15 visit to CeNT.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [h,w] = size(data_set);
% x = zeros(h,w);
% 
% for i = 1:size(data_set,2)
%     row = data_set(:,i);
%     data_set(:,i) = (row-mean(row))/std(row);
% 
%     row = data_set(:,i);
%     x(:,i) = x(:,i) + (row > mean(row) + 0.5);
%     x(:,i) = x(:,i) - (row < mean(row) - 0.5);
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
