function obj = ReprAdaptive_init(D, feature_order)
% Representation with an adaptive threshold ("oh-one" in the old codebase).

obj = struct();

obj.feature_order = (1:199);
if exist('feature_order', 'var')
    obj.feature_order = feature_order;
    fprintf(1, 'Set feature order\n');
end;

% private data members
obj.D = D;

% public data members
obj.name = 'adaptive';
obj.log_name = '_ADAPTIVE_';
obj.len = D + 1;
obj.de_FVr_minbound = zeros(1, D+1);
obj.de_FVr_maxbound = ones(1, D+1);

% public methods
obj.rand_pop = @ReprOhOne_rand_pop;
obj.feature_num = @ReprOhOne_feature_num;
obj.feature_list = @ReprOhOne_feature_list;
obj.gene_list = @ReprOhOne_gene_list;
obj.empty_indiv = @ReprOhOne_empty_indiv;
end

function X = ReprOhOne_rand_pop(obj, N)
X = rand(N, obj.D+1);
end

function flist = ReprOhOne_feature_list(obj, indiv)
thresh = indiv(end);
flist = find(indiv(1:(end-1)) >= thresh);
flist = obj.feature_order(flist);
end

function n = ReprOhOne_feature_num(obj, indiv)
flist = obj.feature_list(obj, indiv);
n = length(flist);
assert(n <= obj.D);
end

function glist = ReprOhOne_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
