function obj = ReprComb_init(wsize, D, feature_order)
% Chromosome consists of three parts: [ real part ][ 0-1 real part ][ threshold ]
% The real part has length wnum and holds feature number for each window
% The 0-1 real part has length wnum and says whether to take the corresponding feature
% from the real part.
% The threshold is a single number taken from 0-1.
% Total length: 2*wnum + 1
%
% Updated 6.12.2016 to accomodate to random embeddings (through feature_order)

obj = struct();

obj.feature_order = (1:199);
if exist('feature_order', 'var')
    obj.feature_order = feature_order;
    fprintf(1, 'Set feature order\n');
end;

% private data members
obj.wsize = wsize;
obj.wnum = ceil(D / wsize);
obj.D = D;

% public data members
obj.name = 'combined';
obj.log_name = strcat('_COMB_', 'wsize=', num2str(wsize), '_');
obj.len = obj.wnum * 2 + 1;
obj.de_FVr_minbound = horzcat(obj.wsize * ((1:obj.wnum) - 0.5), zeros(1, obj.wnum + 1));
obj.de_FVr_maxbound = horzcat(obj.wsize * ((1:obj.wnum) + 0.5), ones(1, obj.wnum + 1));

% public methods
obj.rand_pop = @ReprComb_rand_pop;
obj.feature_num = @ReprComb_feature_num;
obj.feature_list = @ReprComb_feature_list;
obj.gene_list = @ReprComb_gene_list;
obj.empty_indiv = @ReprComb_empty_indiv;

end

function glist = ReprComb_gene_list(obj, indiv)
flist = obj.feature_list(obj, indiv);
end

function X = ReprComb_rand_pop(obj,N)
% First draw the real part.
X1 = obj.wsize * ((repmat(1:obj.wnum, N, 1) - 1) + rand(N, obj.wnum));
% Then draw the 0-1 part + the threshold
X2 = rand(N, obj.wnum + 1);
% Glue together.
X = horzcat(X1, X2);
end

function flist = ReprComb_feature_list(obj, indiv)
inds = round(indiv(1:obj.wnum));
filter = indiv((obj.wnum+1):(2*obj.wnum));
thresh = indiv(end);
if length(inds) ~= length(filter)
    except = MException('ReprComb:feature_list', 'index and feature list len do not match');
    throw(except)
end
% If the highest one is too large, then bring it down.
inds = max(1, min(inds, obj.D));
flist = unique(inds(filter > thresh));
flist = obj.feature_order(flist);
end

function n = ReprComb_feature_num(obj, indiv)
flist = obj.feature_list(obj, indiv);
n = length(flist);
assert(n <= obj.D);
end
