function obj = ReprReal_init(fsize, D, tsne_mapping)
% Chromosome consists of three parts: [ real part ][ 0-1 real part ][ threshold ]
% The real part has length wnum and holds feature number for each window
% The 0-1 real part has length wnum and says whether to take the corresponding feature
% from the real part.
% The threshold is a single number taken from 0-1.
% Total length: 2*wnum + 1

obj = struct();

% private data members
obj.wnum = fsize;
obj.wsize = ceil(D / fsize);
obj.D = D;
obj.tsne_mapping = tsne_mapping;

% public data members
obj.name = 'real';
obj.log_name = strcat('_REAL_', 'fsize=', num2str(fsize), '_');
obj.len = obj.wnum;
obj.de_FVr_minbound = obj.wsize * ((1:obj.wnum) - 0.5);
obj.de_FVr_maxbound = obj.wsize * ((1:obj.wnum) + 0.5);

% public methods
obj.rand_pop = @ReprReal_rand_pop;
obj.feature_num = @ReprReal_feature_num;
obj.feature_list = @ReprReal_feature_list;
obj.gene_list = @ReprReal_gene_list;
obj.empty_indiv = @ReprReal_empty_indiv;

end

function glist = ReprReal_gene_list(obj, indiv)
flist = obj.feature_list(obj, indiv);
% Apply tsne mapping.
glist = obj.tsne_mapping(flist);
end

function X = ReprReal_rand_pop(obj,N)
% First draw the real part.
X = obj.wsize * ((repmat(1:obj.wnum, N, 1) - 1) + rand(N, obj.wnum));
end

function flist = ReprReal_feature_list(obj, indiv)
inds = round(indiv(1:obj.wnum));
% If the highest one is too large, then bring it down.
inds = max(1, min(inds, obj.D));
flist = unique(inds);
end

function n = ReprReal_feature_num(obj, indiv)
flist = obj.feature_list(obj, indiv);
n = length(flist);
assert(n <= obj.D);
end
