function obj = ReprTsne2d_init(wsize, D, embedding)
% Chromosome consists of three parts: [ real part ][ 0-1 real part ][ threshold ]
% The real part has length wnum and holds feature number for each window
% The 0-1 real part has length wnum and says whether to take the corresponding feature
% from the real part.
% The threshold is a single number taken from 0-1.
% Total length: 2*wnum + 1

% Here the values in the first part are original embedding coords.

obj = struct();

% private data members
obj.wsize = wsize;
obj.wnum = ceil(D / wsize);
obj.D = D;
obj.embedding = embedding;

% public data members
obj.name = 'combined-embed';
obj.log_name = strcat('_TSNE2D', 'wsize=', num2str(wsize), '_');
obj.len = obj.wnum * 3 + 1;
obj.de_FVr_minbound = horzcat((1:obj.wnum) * (min(obj.embedding) - 0.5), zeros(1, obj.wnum + 1));
obj.de_FVr_maxbound = horzcat((1:obj.wnum) * (max(obj.embedding) + 0.5), ones(1, obj.wnum + 1));

% public methods
obj.rand_pop = @ReprTsne2d_rand_pop;
obj.feature_num = @ReprTsne2d_feature_num;
obj.feature_list = @ReprTsne2d_feature_list;
obj.gene_list = @ReprTsne2d_gene_list;
obj.empty_indiv = @ReprTsne2d_empty_indiv;

end

function glist = ReprTsne2d_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end

function X = ReprTsne2d_rand_pop(obj,N)
lo = min(obj.embedding) - 0.5;
hi = max(obj.embedding) + 0.5;
len = hi - lo;
% First draw the real part.
X1 = rand(N,obj.wnum * 2) * len + lo;
% Then draw the 0-1 part + the threshold
X2 = rand(N, obj.wnum + 1);
% Glue together.
X = horzcat(X1, X2);
end

function flist = ReprTsne2d_feature_list(obj, indiv)
tsne_v = indiv(1:obj.wnum);
filter = indiv((obj.wnum+1):(2*obj.wnum));
thresh = indiv(end);
inds = find(filter > thresh);
tsne_v = tsne_v(inds);
% Translate tsne_v embedding values to feature nums.
flist = zeros(1, length(tsne_v));
for i = 1:(length(tsne_v))
    [~,ind] = min(abs(obj.embedding - tsne_v(i)));
    flist(i) = ind;
end
% If the highest one is too large, then bring it down.
flist = max(1, min(flist, obj.D));
flist = unique(flist);
end

function n = ReprTsne2d_feature_num(obj, indiv)
filter = indiv((obj.wnum*2+1):end-1);
thresh = indiv(end);
inds = find(filter > thresh);
n = length(inds);
assert(n <= obj.D);
end
