% function [KM_distances, KM_nearest_index, KM_nearest_distance] = kmeans(points, iterations, nclusters)
function cluster_inds = kmeans(points, iterations, nclusters)
% Simple K-Means running for *iterations*, returning nclusters
%
% INPUT: X - matrix of d*N points

[d N] = size(points);
% nclusters = floor(sqrt(N));

centroid_inds = randperm(N);
centroid_inds = centroid_inds(1:nclusters);

KMcenters = points(:, centroid_inds);

for km_i = 1 : iterations
    [KM_distances, KM_nearest_index, KM_nearest_distance] = compute_distances(points, KMcenters);
    for i = 1 : nclusters
        KMcenters(:, i) = mean(points(:, KM_nearest_index == i), 2);
    end
end

% Return the points closest to centroids.
cluster_inds = -1 * ones(nclusters, 1);

nearest_distances = [ KM_nearest_distance ; KM_nearest_index']';
[sorted_nearest_distances sort_inds] = sortrows(nearest_distances);

cluster_dists = zeros(nclusters, 1);

for i = 1:N
    c = sorted_nearest_distances(i, 2);
    if cluster_inds(c) == -1
        cluster_inds(c) = sort_inds(i);
        cluster_dists(c) = sorted_nearest_distances(i, 1);
    end
end
