function obj = ReprManifold4_init(D, manifold, manifold_name)
%
% This representation "walks" over a (t-)SNE manifold.
% Individual is encoded as:
%   [--weights--][--dim1--]...[--dimN--] 
% Each segment has length D <= n_features.
%
% TODO TODO TODO This repr is supposed to assign low pbb to areas with high density,
% and 0 pbb to empty ones during init

if ~exist('manifold_name', 'var')
    manifold_name = 'UNK';
end

obj = struct();

% private data members
[nfeats,mdim] = size(manifold);
obj.D = D;
obj.mdim = mdim; % Manifold dimensionality.

% Rescale manifold dimensions to [0,1] so the vectors would be consistent.
for d = 1:mdim
    col = manifold(1:end, d);
    w = max(col) - min(col);
    col = col - min(col); % Shift to 0
    col = col + abs(w / nfeats); % with a little margin before the first feature
    m_ = max(col) + (2*abs(w / nfeats));
    col = col / m_; % Now col should be within [0,1] with small margins around 0 and 1
    manifold(1:end, d) = col;
end
obj.manifold = manifold;

% public data members
obj.name = strcat('manifold4', num2str(obj.D), manifold_name);
obj.log_name = '_MANIFOLD4_';
obj.len = obj.D*obj.mdim;
obj.de_FVr_minbound = zeros(1, obj.len);
obj.de_FVr_maxbound = ones(1, obj.len);

% public methods
obj.rand_pop = @ReprManifold4_rand_pop;
obj.feature_num = @ReprManifold4_feature_num;
obj.feature_list = @ReprManifold4_feature_list;
obj.gene_list = @ReprManifold4_gene_list;
obj.empty_indiv = @ReprManifold4_empty_indiv;
end

function X = ReprManifold4_rand_pop(obj, N)
% TODO TODO TODO
end

function flist = ReprManifold4_feature_list(obj, indiv)
coords = reshape(indiv, [obj.D, obj.mdim]);
% Find their 1-NNs in the manifold.
dist = mypdist2(obj.manifold, coords);
[~,argmin] = min(dist);
flist = unique(argmin);
end

function n = ReprManifold4_feature_num(obj, indiv)
n = length(obj.feature_list(obj, indiv));
% ind_list = unique(find(indiv(1:obj.D) >= 0.1));
% n = length(ind_list);
% assert(n <= obj.D);
end

function glist = ReprManifold4_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
