function obj = ReprRandManifoldPerm_init(D, manifold, manifold_name)
%
% It uses a 1-D manifold to reorder attributes.
%
% This version differs from previous ones by the fact,
% that it takes the manifold and translates to permuted features.
% Previously the features had to be permuted independently.
%
% Individual is encoded as:
%   [--dim1--]
%

if ~exist('manifold_name', 'var')
    manifold_name = 'UNK';
end

obj = struct();

% private data members
[nfeats, mdim] = size(manifold);
obj.nfeats = nfeats;
obj.D = D;

% Reorder
[~, order] = sort(manifold);  % Order translates to permuted indices
[~, rev_order] = sort(order); % Rev_order gets back indices in the original dataset
obj.order = order;
obj.rev_order = rev_order;
obj.manifold = manifold;

% public data members
obj.name = strcat('randmanifoldperm', manifold_name);
obj.log_name = '_RANDMANIFOLDPERM_';
obj.len = obj.D;
obj.de_FVr_minbound = zeros(1, obj.len) + 0.5;
obj.de_FVr_maxbound = ones(1, obj.len) * obj.nfeats + 0.5;

% public methods
obj.rand_pop = @ReprRandManifoldPerm_rand_pop;
obj.feature_num = @ReprRandManifoldPerm_feature_num;
obj.feature_list = @ReprRandManifoldPerm_feature_list;
obj.gene_list = @ReprRandManifoldPerm_gene_list;
end

function X = ReprRandManifoldPerm_rand_pop(obj, N)
    X = rand(N, obj.len) * (obj.nfeats + 1) - 0.5; % Fit within [0.5, nfeats + 0.5]
end

function flist = ReprRandManifoldPerm_feature_list(obj, indiv)
flist = round(indiv);
flist = min(max(flist,1),obj.nfeats);
flist = unique(flist);
% Apply reverse mapping
flist = obj.rev_order(flist);
end

function n = ReprRandManifoldPerm_feature_num(obj, indiv)
n = length(obj.feature_list(obj, indiv));
end

function glist = ReprRandManifoldPerm_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
