
function main_loop(repr, data_file_ind, data_set, data_labels, param)

% param.max_it - number of iterations of the optimizer
% svm_kernel - which kernel to use:
% 	0 -- linear: u'*v
% 	1 -- polynomial: (gamma*u'*v + coef0)^degree
% 	2 -- radial basis function: exp(-gamma*|u-v|^2)
% 	3 -- sigmoid: tanh(gamma*u'*v + coef0)
% 	4 -- precomputed kernel (kernel values in training_set_file)
%       10 - use built-in adaboost
% k-folds - the number k for k-fold x-validation

rng('shuffle');

%------Global params------------------------------------------------------
N = 100;           % Number of individuals ?
[ntrain, D] = size(data_set);
param.D = D;
param.data_file_ind = data_file_ind;

%------Random initialization of agents.-----------------------------------
X = repr.rand_pop(repr, N);

history = struct;
history.mean_acc = zeros(param.max_it, 1);
history.best_acc = zeros(param.max_it, 1);
history.mean_fnum = zeros(param.max_it, 1);
history.best_fnum = zeros(param.max_it, 1);
history.mean_threshold = zeros(param.max_it, 1);
history.best_indiv = zeros(param.max_it, repr.len);

%----Baseline accuracy.---------------------------------------------------
repr_dummy = ReprDummy_init(D);
dummy_indiv = repr_dummy.rand_pop(repr_dummy, 1); % [ones(1, D) zeros(1, T) ];
if param.svm_kernel < 10
    [fit acc] = inner_loop(dummy_indiv, repr, data_set, data_labels, [], [], param);
else
    [fit acc] = fitness_k_fold(dummy_indiv, repr, data_set, data_labels, param);
end

% XXX
param.base_accuracy = acc;
param.base_fit = fit;

fprintf(1, '---- Baseline accuracy ----\n');
print_best(repr_dummy, 0, dummy_indiv, acc, fit);

%----- CMA-ES ------------------------------------------------------------
fprintf(1, 'Running demo with (full) CMA-ES optimizer\n');

% xmin      % minimum search point of last iteration
% fmin      % function value of xmin
% counteval % number of function evaluations done
% stopflag  % stop criterion reached
% out       % struct with various histories and solutions
% bestever  % struct containing overall best solution (for convenience)
%
% fitfun    % name of objective/fitness function
% xstart    % objective variables initial point, determines N
% insigma   % initial coordinate wise standard deviation(s)
% inopts    % options struct, see defopts below
% varargin  % arguments passed to objective function 

% Options defaults: Stopping criteria % (value of stop flag)
defopts.StopFitness  = '-Inf % stop if f(xmin) < stopfitness, minimization';
defopts.MaxFunEvals  = 'Inf  % maximal number of fevals';
% defopts.MaxIter      = '1e3*(N+5)^2/sqrt(popsize) % maximal number of iterations';
defopts.MaxIter      = param.max_it; '200 % maximal number of iterations';
defopts.StopFunEvals = 'Inf  % stop after resp. evaluation, possibly resume later';
defopts.StopIter     = 'Inf  % stop after resp. iteration, possibly resume later';
defopts.TolX         = '0.0 % stop if x-change smaller TolX';
% defopts.TolX         = '1e-11*max(insigma) % stop if x-change smaller TolX';
defopts.TolUpX       = 'Inf % stop if x-changes larger TolUpX';
% defopts.TolUpX       = '1e3*max(insigma) % stop if x-changes larger TolUpX';
defopts.TolFun       = '0.0 % stop if fun-changes smaller TolFun';
% defopts.TolFun       = '1e-12 % stop if fun-changes smaller TolFun';
defopts.TolHistFun   = '0.0 % stop if back fun-changes smaller TolHistFun';
% defopts.TolHistFun   = '1e-13 % stop if back fun-changes smaller TolHistFun';
defopts.StopOnStagnation = 'off  % stop when fitness stagnates for a long time'; % Adrian
% TODO: stagnation has four parameters for the period: min = 120, const = 30N/lam, rel = 0.2, max = 2e5
% defopts.StopOnStagnation = '[120 30*N/popsize 0.2 2e5]  % [min const rel_iter max] measuring period';
defopts.StopOnWarnings = 'no   % ''no''==''off''==0, ''on''==''yes''==1 ';
defopts.StopOnEqualFunctionValues = 'Inf % number of iterations';  
% defopts.StopOnEqualFunctionValues = '2 + N/3  % number of iterations';  

% Options defaults: Other
defopts.DiffMaxChange = 'Inf  % maximal variable change(s), can be Nx1-vector';
defopts.DiffMinChange = '0    % minimal variable change(s), can be Nx1-vector';
defopts.WarnOnEqualFunctionValues = ...
    'yes  % ''no''==''off''==0, ''on''==''yes''==1 ';
defopts.LBounds = repr.de_FVr_minbound'; % lower bounds, scalar or Nx1-vector';  % Adrian
defopts.UBounds = repr.de_FVr_maxbound'; % upper bounds, scalar or Nx1-vector';  % Adrian
defopts.EvalParallel = 'yes  % objective function FUN accepts NxM matrix, with M>1?';
defopts.EvalInitialX = 'yes  % evaluation of initial solution';
defopts.Restarts     = '0    % number of restarts ';
defopts.IncPopSize   = '2    % multiplier for population size before each restart';

defopts.PopSize      = N;
defopts.ParentNumber = 'floor(popsize/2)       % AKA mu, popsize equals lambda';
defopts.RecombinationWeights = 'superlinear decrease % or linear, or equal';
defopts.DiagonalOnly = '0*(1+100*N/sqrt(popsize))+(N>=1000)  % C is diagonal for given iterations, 1==always'; 
defopts.Noise.on = '0  % uncertainty handling is off by default'; 
defopts.Noise.reevals  = '1*ceil(0.05*lambda)  % nb. of re-evaluated for uncertainty measurement';
defopts.Noise.theta = '0.5  % threshold to invoke uncertainty treatment'; % smaller: more likely to diverge
defopts.Noise.cum = '0.3  % cumulation constant for uncertainty'; 
defopts.Noise.cutoff = '2*lambda/3  % rank change cutoff for summation';
defopts.Noise.alphasigma  = '1+2/(N+10)  % factor for increasing sigma'; % smaller: slower adaptation
defopts.Noise.epsilon  = '1e-7  % additional relative perturbation before reevaluation';
defopts.Noise.minmaxevals = '[1 inf]  % min and max value of 2nd arg to fitfun, start value is 5th arg to cmaes'; 
defopts.Noise.alphaevals  = '1+2/(N+10)  % factor for increasing 2nd arg to fitfun'; 
defopts.Noise.callback = '[]  % callback function when uncertainty threshold is exceeded';
% defopts.TPA = 0; 
defopts.CMA.cs = '(mueff+2)/(N+mueff+3)  % cumulation constant for step-size'; 
   %qqq defopts.CMA.cs = (mueff^0.5)/(N^0.5+mueff^0.5) % the short time horizon version
defopts.CMA.damps = '1 + 2*max(0,sqrt((mueff-1)/(N+1))-1) + cs  % damping for step-size';
% defopts.CMA.ccum = '4/(N+4)  % cumulation constant for covariance matrix'; 
defopts.CMA.ccum = '(4 + mueff/N) / (N+4 + 2*mueff/N)  % cumulation constant for pc';
defopts.CMA.ccov1 = '2 / ((N+1.3)^2+mueff)  % learning rate for rank-one update'; 
defopts.CMA.ccovmu = '2 * (mueff-2+1/mueff) / ((N+2)^2+mueff) % learning rate for rank-mu update'; 
defopts.CMA.on     = 'yes'; 
defopts.CMA.active = '0  % active CMA, 1: neg. updates with pos. def. check, 2: neg. updates'; 

defopts.DispModulo = '0  % [0:Inf], disp messages after every i-th iteration'; % TODO verbose
defopts.LogModulo = '0    % [0:Inf] if >1 record data less frequently after gen=100'; % Do not save .dat files in the main dir

%------------------------------------------------------

% xstart = rand(10,1); % 0.50*ones(10,1);
insigma = 0.3;
inopts = defopts;

xstart = X(1,:);

[xmin, fmin, counteval, stopflag, out, bestever, history] = cmaes('myfitness', xstart, insigma, inopts, history, repr, data_set, data_labels, param);
save_log(repr, param, history, struct(), -1); % XXX XXX XXX iter = 0
